/***************************************************************************
                          kplayer.h
                          ---------
    begin                : Sat Nov 16 10:12:50 EST 2002
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KPLAYER_H
#define KPLAYER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kaction.h>
#include <kmainwindow.h>
#include <kuniqueapplication.h>

#include "kplayerprocess.h"
#include "kplayerwidget.h"

class KCmdLineArgs;
class KPlayerLogWindow;
class KPlayerPlaylist;
class KPlayerPlaylistAction;
class KPlayerPlaylistWindow;
class KPlayerPopupSliderAction;
class KPlayerSettings;
class KPlayerSliderAction;
class KRecentFilesAction;
class KStatusBarLabel;
class KToggleAction;
class KURL;

#define MAIN_TOOLBAR         0
#define EDITOR_TOOLBAR       1
#define PLAYLIST_TOOLBAR     2
#define FIRST_SLIDER_TOOLBAR 3
#define PROGRESS_TOOLBAR     3
#define VOLUME_TOOLBAR       4
#define FIRST_VIDEO_TOOLBAR  5
#define CONTRAST_TOOLBAR     5
#define BRIGHTNESS_TOOLBAR   6
#define HUE_TOOLBAR          7
#define SATURATION_TOOLBAR   8
#define KPLAYER_TOOLBARS     9

/** The KPlayer main window class.
  *@author kiriuja
  */
class KPlayer : public KMainWindow
{
  Q_OBJECT 

  typedef struct _Toolbar
  {
    const char* name;
    const char* action;
    bool normally_visible;
    bool fullscreen_visible;
  } Toolbar;

public:
  /** Constructs the KPlayer main window. Parameters are passed to KMainWindow constructor.
    */
  KPlayer (QWidget* parent=0, const char *name=0);
  /** KPlayer main window destructor. Does nothing.
    */
  virtual ~KPlayer() { }

  /** Returns the KPlayerProcess object.
    */
  KPlayerProcess* kPlayerProcess (void) const
    { return kPlayerWidget() -> kPlayerProcess(); }
  /** Returns the KPlayerWidget object.
    */
  KPlayerWidget* kPlayerWidget (void) const
    { return kPlayerWorkspace() -> kPlayerWidget(); }
  /** Returns the KPlayerWorkspace object.
    */
  KPlayerWorkspace* kPlayerWorkspace (void) const
    { return (KPlayerWorkspace*) centralWidget(); }

  /** Starts playing a new file.
    */
  void start (void);
  /** Adds the given URL to the playlist, loads it and optionally starts playback.
    */
  void openUrl (KURL);
  /** Adds the given list of URLs to the playlist, loads it and optionally starts playback.
    */
  void openUrls (KURL::List);
  /** Loads the given URL and optionally starts playback.
    */
  void loadUrl (KURL, bool play = true, bool immediately = false);
  /** Loads the given subtitle URL and restarts playback if needed.
   */
  void loadSubtitle (KURL);
  /** Zooms the video to the correct size.
    */
  void zoom (bool user_zoom = true);
  /** Logs a message if logging of KPlayer messages is enabled.
   */
  void log (QString);

  /** Shows or hides the menu bar according to the current settings and updates the user interface.
   */
  void showMenubar (void);
  /** Shows or hides the given toolbar according to the current settings and updates the user interface.
   */
  void showToolbar (int);
  /** Shows or hides the given toolbar and updates the user interface.
   */
  void showToolbar (int, bool);
  /** Shows or hides the status bar according to the current settings and updates the user interface.
   */
  void showStatusbar (void);
  /** Shows or hides the message log according to the current settings and updates the user interface.
   */
  void showMessageLog (void);
  /** Shows or hides the message log and updates the user interface.
   */
  void showMessageLog (bool);

  /** Calculates the size hint for the main window.
   */
  //virtual QSize sizeHint (void) const;
  /** Calculates the minimum size hint for the main window.
   */
  virtual QSize minimumSizeHint (void) const;
  /** Sets the minimum size of the main widget.
   */
  virtual void setMinimumSize (int, int);
  /** Returns whether the main window is full screen.
   */
  bool isFullScreen (void);

protected:
  /** Initializes the KActions of the application.
    */
  void initActions (void);
  /** Sets up the status bar.
    */
  void initStatusBar (void);

  /** Starts playback.
    */
  void playerProcessPlay (void);

  /** Save general options like all bar positions and status as well as the geometry
      and the recent file list to the configuration file. */
  void saveOptions (void);
  /** Read general options again and initialize all variables like the recent file list.
    */
  void readOptions (void);
  /** Saves the current URLs during session end to the session config file.
      @see KTMainWindow#saveProperties */
  virtual void saveProperties (KConfig*) const;
  /** Restores the URLs from the session config file.
      @see KTMainWindow#readProperties */
  virtual void readProperties (KConfig*);

  /** Disables screen saver if it is enabled.
   */
  void disableScreenSaver (void);
  /** Enables screen saver if it was disabled.
   */
  void enableScreenSaver (void);

  /** Enables or disables player actions.
    */
  void enablePlayerActions (void);
  /** Enables or disables video actions.
    */
  void enableVideoActions (void);
  /** Enables or disables subtitle actions.
    */
  void enableSubtitleActions (void);
  /** Enables or disables, checks or unchecks zoom actions.
    */
  void enableZoomActions (void);
  /** Checks or unchecks bar actions. */
//void checkBarActions (void);
  /** Enables or disables a submenu on the menu bar.
   */
  void enableSubmenu (int index, int subindex, bool enable);

  /** Enables a toolbar.
   * @param index the toolbar number
   */
  void enableToolbar (int index);
  /** Disables a toolbar.
   * @param index the toolbar number
   */
  void disableToolbar (int index);

  /** Sets the sound volume.
    */
  void setVolume (int);
  /** Sets the video contrast.
    */
  void setContrast (int);
  /** Sets the video brightness.
    */
  void setBrightness (int);
  /** Sets the video hue.
    */
  void setHue (int);
  /** Sets the video saturation.
    */
  void setSaturation (int);

  /** Changes the statusbar contents for the standard label permanently, used to indicate current actions.
   */
  void setStatusText (const QString&);

  /** Processes the widget events. Passes them on to KMainWindow. */
  //virtual bool event (QEvent*);
  /** Sets the window geometry on the initial show event. */
  virtual void showEvent (QShowEvent*);
  /** Handles main window resizing. */
  virtual void resizeEvent (QResizeEvent*);
  /** Displays the right click popup menu.
    */
  virtual void contextMenuEvent (QContextMenuEvent*);
  /** Handles the drag enter event.
   */
  virtual void dragEnterEvent (QDragEnterEvent*);
  /** Handles the drop event.
   */
  virtual void dropEvent (QDropEvent*);

  /** Retrieves an action from the actionCollection by name.
    */
  KAction* action (const char* name) const
    { return actionCollection() -> action (name); }
  /** Retrieves a slider action from the actionCollection by name.
    */
  KPlayerSliderAction* sliderAction (const char* name) const
    { return (KPlayerSliderAction*) action (name); }
  /** Retrieves a slider action from the actionCollection by name.
    */
  KPlayerPopupSliderAction* popupAction (const char* name) const
    { return (KPlayerPopupSliderAction*) action (name); }
  /** Retrieves a toggle action from the actionCollection by name.
    */
  KToggleAction* toggleAction (const char* name) const
    { return (KToggleAction*) action (name); }
  /** Retrieves the recent files action from the actionCollection.
    */
  KRecentFilesAction* recentFilesAction (void) const
    { return (KRecentFilesAction*) action (KStdAction::stdName (KStdAction::OpenRecent)); }
  /** Retrieves the playlist action from the actionCollection.
    */
  KPlayerPlaylistAction* playlistAction (void) const
    { return (KPlayerPlaylistAction*) action ("playlist_list"); }

  /** The log window object.
   */
  KPlayerLogWindow* m_log;
  /** The playlist window object.
   */
  KPlayerPlaylistWindow* m_playlist;

  // Following should be private.
  bool m_resizing, m_updating, m_initial_fullscreen, m_enable_screen_saver;
  bool m_menubar_normally_visible, m_menubar_fullscreen_visible;
  bool m_statusbar_normally_visible, m_statusbar_fullscreen_visible;
  bool m_messagelog_normally_visible, m_messagelog_fullscreen_visible;
  bool m_editor_normally_visible, m_editor_fullscreen_visible;
  bool m_stop, m_loading_url, m_initial_show, m_error_detected, m_full_screen;
  bool m_delayed_size;
  Toolbar m_toolbar [KPLAYER_TOOLBARS];
  QSize m_previous_size, m_video_size;
  QRect m_normal_geometry;
  KStatusBarLabel *m_status_label, *m_state_label, *m_progress_label;
  int m_progress_factor; // work around QRangeControl bug

public slots:
  /** Stops the KPlayerProcess and closes the KPlayerWidget.
    */
  virtual void closeEvent (QCloseEvent*);
  /** Displays the playlist context menu.
   */
  void playlistContextMenu (KListView*, QListViewItem*, const QPoint&);

  /** Displays the Open File dialog and starts playing the chosen file.
    */
  void fileOpen (void);
  /** Displays the Open URL dialog and starts playing from the entered URL.
    */
  void fileOpenUrl (void);
  /** Displays the Open Subtitle File dialog and loads the chosen subtitles.
      Restarts playback if a file was playing.
    */
  void fileOpenSubtitles (void);
  /** Displays the Open Subtitle URL dialog and loads the chosen subtitles.
      Restarts playback if a file was playing.
    */
  void fileOpenSubtitleUrl (void);
  /** Unloads subtitles and prevents subtitle autoloading.
      Restarts playback if a file was playing.
    */
  void fileUnloadSubtitles (void);
  /** Opens a file from the recent files menu.
    */
  void fileOpenRecent (const KURL&);
  /** Clears all messages from the message log.
    */
  void fileClearLog (void);
  /** Ends the program by closing the main window.
    */
  void fileQuit (void);

  /** Toggles full screen mode.
    */
  void viewFullScreen (void);
  /** Scales video up.
    */
  void viewZoomIn (void);
  /** Scales video down.
    */
  void viewZoomOut (void);
  /** Scales video to 50%.
    */
  void viewZoom12 (void);
  /** Scales video to 100%.
    */
  void viewZoom11 (void);
  /** Scales video to 150%.
    */
  void viewZoom32 (void);
  /** Scales video to 200%.
    */
  void viewZoom21 (void);
  /** Scales video to 250%.
    */
  void viewZoom52 (void);
  /** Scales video to 300%.
    */
  void viewZoom31 (void);
  /** Toggles the option of maintaining the original video aspect ratio.
    */
  void viewMaintainOriginalAspect (void);
  /** Toggles the option of maintaining the current video aspect ratio.
    */
  void viewMaintainCurrentAspect (void);
  /** Forces 4 to 3 video aspect ratio.
    */
  void viewAspect43 (void);
  /** Forces 16 to 9 video aspect ratio.
    */
  void viewAspect169 (void);

  /** Plays the currently loaded file.
    */
  void playerPlay (void);
  /** Pauses the player.
    */
  void playerPause (void);
  /** Stops the player.
    */
  void playerStop (void);
  /** Seeks forward.
    */
  void playerForward (void);
  /** Seeks forward fast.
    */
  void playerFastForward (void);
  /** Seeks backward.
    */
  void playerBackward (void);
  /** Seeks backward fast.
    */
  void playerFastBackward (void);
  /** Seeks to the beginning.
    */
  void playerStart (void);
  /** Increases the volume.
    */
  void playerVolumeUp (void);
  /** Decreases the volume.
    */
  void playerVolumeDown (void);
  /** Toggles audio playback.
    */
  void playerMute (void);
  /** Toggle the soft frame dropping option.
    */
  void playerSoftFrameDrop (void);
  /** Toggle the hard frame dropping option.
    */
  void playerHardFrameDrop (void);
  /** Shows or hides subtitles.
    */
  void playerSubtitlesShow (void);
  /** Moves subtitles down.
    */
  void playerSubtitlesMoveDown (void);
  /** Moves subtitles up.
    */
  void playerSubtitlesMoveUp (void);
  /** Decreases subtitle delay.
    */
  void playerSubtitlesDelayDecrease (void);
  /** Increases subtitle delay.
    */
  void playerSubtitlesDelayIncrease (void);

  /** Shows or hides the playlist.
    */
  void playlistShow (void);
  /** Shows or hides the playlist editor according to the current settings and updates the user interface.
   */
  void showPlaylistEditor (void);
  /** Shows or hides the playlist editor and updates the user interface.
   */
  void showPlaylistEditor (bool);
  /** Shows or hides the playlist editor.
   */
  void playlistEditor (void);
  /** Shows or hides the playlist editor toolbar.
    */
  void playlistToolbar (void);
  /** Plays the current item on the playlist.
   */
  void playlistPlay (void);
  /** Plays the next item on the playlist.
   */
  void playlistNext (void);
  /** Plays the previous item on the playlist.
   */
  void playlistPrevious (void);
  /** Toggles the option to play selection only or the entire playlist.
   */
  void playlistSelection (void);
  /** Locks the playlist so no new items are added or unlocks it.
   */
  void playlistLock (void);
  /** Toggles the option to loop through the list.
   */
  void playlistLoop (void);
  /** Toggles the option to play items in random order.
   */
  void playlistShuffle (void);

  /** Increases the video brightness.
    */
  void videoBrightnessUp (void);
  /** Decreases the video brightness.
    */
  void videoBrightnessDown (void);
  /** Increases the video contrast.
    */
  void videoContrastUp (void);
  /** Decreases the video contrast.
    */
  void videoContrastDown (void);
  /** Increases the video hue.
    */
  void videoHueUp (void);
  /** Decreases the video hue.
    */
  void videoHueDown (void);
  /** Increases the video saturation.
    */
  void videoSaturationUp (void);
  /** Decreases the video saturation.
    */
  void videoSaturationDown (void);

  /** Toggles the menu bar.
    */
  void viewMenubar (void);
  /** Toggles the status bar.
    */
  void viewStatusbar (void);
  /** Toggles the message log.
    */
  void viewMessageLog (void);
  /** Toggles the main toolbar.
    */
  void viewMainToolbar (void);
  /** Toggles the progress toolbar.
    */
  void viewProgressToolbar (void);
  /** Toggles the volume toolbar.
    */
  void viewVolumeToolbar (void);
  /** Toggles the brightness toolbar.
    */
  void viewBrightnessToolbar (void);
  /** Toggles the contrast toolbar.
    */
  void viewContrastToolbar (void);
  /** Toggles the hue toolbar.
    */
  void viewHueToolbar (void);
  /** Toggles the Saturation toolbar.
    */
  void viewSaturationToolbar (void);
  /** Opens the the standard key binding dialog.
    */
  void settingsKeyBindings (void);
  /** Opens the KPlayer configuration dialog.
    */
  void settingsConfigure (void);

  /** Moves to the position chosen.
    */
  void progressChanged (int);
  /** Changes the volume level.
    */
  void volumeChanged (int);
  /** Changes the video brightness.
    */
  void brightnessChanged (int);
  /** Changes the video contrast.
    */
  void contrastChanged (int);
  /** Changes the video hue.
    */
  void hueChanged (int);
  /** Changes the video saturation.
    */
  void saturationChanged (int);

  /** Receives the stateChanged signal from KPlayerProcess.
    */
  void playerStateChanged (KPlayerProcess::State);
  /** Receives the progressChanged signal from KPlayerProcess.
    */
  void playerProgressChanged (float, KPlayerProcess::ProgressType);
  /** Receives the infoAvailable signal from KPlayerProcess.
    */
  void playerInfoAvailable (void);
  /** Receives the sizeAvailable signal from KPlayerProcess.
    */
  void playerSizeAvailable (void);
  /** Receives the messageReceived signal from KPlayerProcess.
   */
  void playerMessageReceived (const char*);
  /** Receives the errorDetected signal from KPlayerProcess.
   */
  void playerErrorDetected (void);
  /** Receives the windowHidden signal from KPlayerLogWindow. Updates the menu item.
   */
  void logWindowHidden (void);
  /** Receives the windowHidden signal from KPlayerPlaylistWindow. Updates the menu item.
   */
  void playlistEditorHidden (void);
  /** Receives the saved signal from KPlayerSettings. Updates the slider settings.
   */
  void refreshSettings (void);

  /** Changes the statusbar test temporarily, used to display a hint on the currently highlighted action.
   */
  void setStatusMessage (const QString&);
  /** Clears the temporary statusbar text when the action is no longer highlighted.
    */
  void clearStatusMessage (void);

  /** Handles widget resized event.
    */
  void widgetResized (void);

  /** Handles bar orientation change event. */
//void barOrientationChanged (Orientation);
  /** Handles bar place change event. */
//void barPlaceChanged (QDockWindow::Place);
  /** Handles bar visibility change event. */
//void barVisibilityChanged (bool);
};

/** The KPlayer application class.
  *@author kiriuja
  */
class KPlayerApplication : public KUniqueApplication
{
  Q_OBJECT 

public:
  /** Constructs the KPlayer application.
    */
  KPlayerApplication (void);
  /** KPlayer application destructor. Deletes the settings object if it had been created.
    */
  virtual ~KPlayerApplication();

  /** Creates the main window on the first invokation. Calls KPlayer::start().
    */
  virtual int newInstance (void);
};

#endif
