/***************************************************************************
                          kplayerplaylist.h
                          ------------------
    begin                : Wed Sep 3 2003
    copyright            : (C) 2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KPLAYERPLAYLIST_H
#define KPLAYERPLAYLIST_H

#include <kaction.h>
#include <kcombobox.h>
#include <klistview.h>
#include <qdockwindow.h>

#include "kplayerprocess.h"

class KPlayerPlaylistWidget;
class KPlayerProcess;

/**Playlist item class, contains the playlist item data.
  *@author kiriuja
  */
class KPlayerPlaylistItem
{
public:
  /** The KPlayerPlaylistItem constructor. Adds a new list item to the list with the given URL.
    */
  KPlayerPlaylistItem (const KURL&);
  /** The KPlayerPlaylistItem destructor. Saves the playlist item data.
    */
  virtual ~KPlayerPlaylistItem();

  /** Returns the URL of the playlist entry.
   */
  KURL url (void) const
    { return m_url; }
  /** Sets the URL of the playlist entry.
   */
  void setUrl (const KURL& url)
    { m_url = url; }

  /** Returns the name of the playlist entry.
   */
  QString name (void) const
    { return m_name; }
  /** Sets the name of the playlist entry.
   */
  virtual void setName (const QString& name);

  /** Returns the time length.
   */
  float length (void) const
    { return m_length; }
  /** Sets the time length.
   */
  virtual void setLength (float length)
    { m_length = length; }

  /** Returns the video size.
   */
  QSize videoSize (void) const
    { return m_video_size; }
  /** Sets the video size.
   */
  virtual void setVideoSize (const QSize& video_size)
    { m_video_size = video_size; }

  /** Returns the video codec.
   */
  QString videoCodec (void) const
    { return m_video_codec; }
  /** Sets the video codec.
   */
  virtual void setVideoCodec (const QString& video_codec)
    { m_video_codec = video_codec; }

  /** Returns the video bitrate.
   */
  int videoBitrate (void) const
    { return m_video_bitrate; }
  /** Sets the video bitrate.
   */
  virtual void setVideoBitrate (int video_bitrate)
    { m_video_bitrate = video_bitrate; }

  /** Returns the audio codec.
   */
  QString audioCodec (void) const
    { return m_audio_codec; }
  /** Sets the audio codec.
   */
  virtual void setAudioCodec (const QString& audio_codec)
    { m_audio_codec = audio_codec; }

  /** Returns the audio bitrate.
   */
  int audioBitrate (void) const
    { return m_audio_bitrate; }
  /** Sets the audio bitrate.
   */
  virtual void setAudioBitrate (int audio_bitrate)
    { m_audio_bitrate = audio_bitrate; }

  /** Loads the item data from playlist config.
   */
  virtual void load (void);

protected:
  /** The URL of the playlist entry.
   */
  KURL m_url;
  /** The name of the playlist entry, defaults to the file name.
   */
  QString m_name;
  /** The time length.
   */
  float m_length;
  /** The video size.
   */
  QSize m_video_size;
  /** The video codec.
   */
  QString m_video_codec;
  /** The video bitrate in kbps.
   */
  int m_video_bitrate;
  /** The audio codec.
   */
  QString m_audio_codec;
  /** The audio bitrate in kbps.
   */
  int m_audio_bitrate;
};

/**Playlist class, contains the list of playlist items.
  *@author kiriuja
  */
class KPlayerPlaylist : protected QPtrList<KPlayerPlaylistItem>
{
public:
  /** The KPlayerPlaylist constructor. Creates an empty playlist.
    */
  KPlayerPlaylist (void);
  /** The KPlayerPlaylist destructor.
    */
  virtual ~KPlayerPlaylist();

  /** Returns whether the playlist is empty.
   */
  bool isEmpty (void)
   { return QPtrList<KPlayerPlaylistItem>::isEmpty(); }
  /** Returns the number of items on the playlist.
   */
  int count (void)
   { return QPtrList<KPlayerPlaylistItem>::count(); }
  /** Returns whether the given item is on the playlist.
   */
  bool has (KPlayerPlaylistItem*);
  /** Returns whether the playlist is temporary and has autodelete option set.
   */
  bool isTemporary (void)
   { return autoDelete(); }
  /** Sets the playlist to temporary and also sets the autodelete option.
   */
  void setTemporary (bool temporary)
   { setAutoDelete (temporary); }

  /** Returns whether the playlist loops.
   */
  bool loop (void)
   { return m_loop; }
  /** Sets the playlist loop option.
   */
  void setLoop (bool loop)
   { m_loop = loop; }
  /** Returns whether the playlist is shuffled.
   */
  bool shuffle (void)
   { return m_shuffle; }
  /** Sets the playlist shuffle option and shuffles the playlist if the option is on.
   */
  void setShuffle (bool shuffle);

  /** Returns the index of the current item or -1 when at the list end.
   */
  int index (void)
    { return QPtrList<KPlayerPlaylistItem>::at(); }
  /** Returns the current playlist item.
   */
  KPlayerPlaylistItem* current (void)
    { return QPtrList<KPlayerPlaylistItem>::current(); }
  /** Returns the playlist item with the given index.
   */
  KPlayerPlaylistItem* item (int index)
    { return at (index); }
  /** Sets the current playlist item.
   */
  virtual void setCurrent (KPlayerPlaylistItem* item);
  /** Sets the current playlist item.
   */
  void setCurrent (int index);

  /** Goes to the next item on the playlist.
   */
  virtual bool next (void);
  /** Goes to the previous item on the playlist.
   */
  virtual bool previous (void);
  /** Removes all items from the playlist.
   */
  virtual void clear (void);

  /** Adds the given item to the end of the playlist.
    */
  virtual void add (KPlayerPlaylistItem*);
  /** Removes the given item from the playlist.
    */
  virtual void remove (KPlayerPlaylistItem*);
  /** Moves the given item after the other given item.
    */
  virtual void move (KPlayerPlaylistItem*, KPlayerPlaylistItem*);
  /** Renames the given item with the given name.
   */
  virtual void rename (KPlayerPlaylistItem*, const QString&);

  /** Shuffles the playlist entries.
   */
  virtual void randomize (KPlayerPlaylistItem* item = 0);

protected:
  /** The loop option.
   */
  bool m_loop;
  /** The shuffle option.
   */
  bool m_shuffle;
};

/**The playlist combobox widget.
  *@author kiriuja
  */
class KPlayerPlaylistCombobox : public KComboBox, public KPlayerPlaylist
{
  Q_OBJECT

public:
  /** The KPlayerPlaylistCombobox constructor.
    */
  KPlayerPlaylistCombobox (QWidget* parent = 0, const char* name = 0);

  /** Sets the current playlist item.
   */
  virtual void setCurrent (KPlayerPlaylistItem* item);

  /** Goes to the next item on the playlist.
   */
  virtual bool next (void);
  /** Goes to the previous item on the playlist.
   */
  virtual bool previous (void);
  /** Removes all items from the playlist.
   */
  virtual void clear (void);

  /** Adds the given item to the end of the playlist.
    */
  virtual void add (KPlayerPlaylistItem*);
  /** Removes the given item from the playlist.
    */
  virtual void remove (KPlayerPlaylistItem*);
  /** Moves the given item after the other given item.
    */
  virtual void move (KPlayerPlaylistItem*, KPlayerPlaylistItem*);
  /** Renames the given item with the given name.
   */
  virtual void rename (KPlayerPlaylistItem*, const QString&);

  /** Shuffles the playlist entries.
   */
  virtual void randomize (KPlayerPlaylistItem* item = 0);

  /** The size hint.
   */
  virtual QSize sizeHint() const;
  /** The minimum size hint.
   */
  virtual QSize minimumSizeHint() const;
};

/**Playlist view item class, good for insertion into a list view.
  *@author kiriuja
  */
class KPlayerPlaylistViewItem : public KListViewItem, public KPlayerPlaylistItem
{
public:
  /** The KPlayerPlaylistViewItem constructor. Adds a new list item to the list with the given URL.
    */
  KPlayerPlaylistViewItem (KPlayerPlaylistWidget*, const KURL&);
  /** The KPlayerPlaylistViewItem destructor.
    */
  virtual ~KPlayerPlaylistViewItem();

  /** Sets the name of the playlist entry.
   */
  virtual void setName (const QString& name);
  /** Sets the time length.
   */
  virtual void setLength (float length);
  /** Sets the video size.
   */
  virtual void setVideoSize (const QSize& video_size);
  /** Sets the video codec.
   */
  virtual void setVideoCodec (const QString& video_codec);
  /** Sets the video bitrate.
   */
  virtual void setVideoBitrate (int video_bitrate);
  /** Sets the audio codec.
   */
  virtual void setAudioCodec (const QString& audio_codec);
  /** Sets the audio bitrate.
   */
  virtual void setAudioBitrate (int audio_bitrate);

  /** Loads the item data from playlist config.
   */
  virtual void load (void);
};

/**Playlist combobox action suitable for insertion into a toolbar.
  *@author kiriuja
  */
class KPlayerPlaylistAction : public KWidgetAction
{
  Q_OBJECT

public:
  /** The KPlayerPlaylistAction constructor. Parameters are passed on to KAction.
    */
  KPlayerPlaylistAction (const QString& text, const KShortcut&, const QObject* receiver,
    const char* slot, KActionCollection* parent = 0, const char* name = 0);
  /** The KPlayerPlaylistAction destructor. Does nothing.
    */
  virtual ~KPlayerPlaylistAction();

  /** Returns a pointer to the playlist combobox object.
    */
  KPlayerPlaylistCombobox* playlist (void)
    { return (KPlayerPlaylistCombobox*) widget(); }

  /** Plugs the slider into the toolbar.
    */
  //virtual int plug (QWidget* widget, int index = -1);
  /** Unplugs the slider from the toolbar.
    */
  //virtual void unplug (QWidget* widget);

public slots:
  /** Selects the given item and emits the play signal.
   */
  void play (int index);
};

/**The KPlayer playlist widget.
  *@author kiriuja
  */
class KPlayerPlaylistWidget : public KListView
{
   Q_OBJECT

public: 
  KPlayerPlaylistWidget (QWidget* parent = 0, const char* name = 0);
  virtual ~KPlayerPlaylistWidget();

  KPlayerPlaylist* playlist (void)
   { return m_list; }
  void setPlaylist (KPlayerPlaylist* list)
   { m_list = list; }
  KActionCollection* actionCollection (void)
   { return m_ac; }
  void setActionCollection (KActionCollection* collection)
   { m_ac = collection; }

  /** Returns the URL of the current item on the playlist or an empty
      URL if at the list end. */
  KURL current (void);
  /** Returns the URL of the next item on the playlist or an empty
      URL if at the list end. */
  KURL next (void);
  /** Returns the URL of the previous item on the playlist or an empty
      URL if at the list end. */
  KURL previous (void);

  /** Returns whether the playlist is locked.
   */
  bool locked (void)
   { return m_locked; }
  /** Sets the option to lock the playlist and not accept more items.
   */
  void setLocked (bool locked)
   { m_locked = locked; }
  /** Returns whether only the current selection is played.
   */
  bool selectionOnly (void)
   { return m_selection_only; }
  /** Sets the option to play only the current selection.
   */
  void setSelectionOnly (bool selection_only);
  /** Returns whether the playlist loops.
   */
  bool loop (void)
   { return m_list -> loop(); }
  /** Sets the playlist loop option.
   */
  void setLoop (bool loop);
  /** Returns whether the playlist is shuffled.
   */
  bool shuffle (void)
   { return m_list -> shuffle(); }
  /** Sets the playlist shuffle option and reloads the playlist if the option is off.
   */
  void setShuffle (bool shuffle);

  /** Save playlist contents and options to the configuration file.
   */
  void saveOptions (void);
  /** Read playlist contents and options and initialize the playlist.
    */
  void readOptions (void);

  /** Enables or disables the next and previous actions according to the current state.
    */
  void enableNextPrevious (bool same_url = false);
  /** Enables or disables playlist actions according to the current state.
    */
  void enableActions (void);

  /** Sets selection of the current item and updates playlist if the
      selectionOnly option is set. */
  //virtual void setSelected (QListViewItem* item, bool selected);

public slots:
  /** Starts playing the current item.
   */
  void play (void);
  /** Starts playing the given item.
   */
  void play (QListViewItem*);
  /** Starts renaming the current item.
   */
  void rename (void);
  /** Renames the given item.
   */
  virtual void rename (QListViewItem*, int);
  /** Renames the given item.
   */
  void rename (QListViewItem*, const QString&, int);
  /** Moves the given item.
   */
  void move (QListViewItem*, QListViewItem*, QListViewItem*);
  /** Adds a single item to the playlist.
   */
  void add (const KURL&);
  /** Adds items to the playlist.
   */
  void add (KURL::List&);
  /** Adds items to the playlist after the given item.
   */
  void add (KURL::List&, KPlayerPlaylistViewItem*);
  /** Moves the currently selected items up in the playlist.
   */
  void moveUp (void);
  /** Moves the currently selected items down in the playlist.
   */
  void moveDown (void);
  /** Removes the currently selected items from the playlist.
   */
  void remove (void);
  /** Removes all items from the playlist.
   */
  virtual void clear (void);

  /** Receives the stateChanged signal from KPlayerProcess.
    */
  void playerStateChanged (KPlayerProcess::State);
  /** Receives the infoAvailable signal from KPlayerProcess.
    */
  void playerInfoAvailable (void);

  /** Sets selection of the current item and updates playlist if the
      selectionOnly option is set. */
  void updateSelection (void);

  /** Adjusts the playlist to the user preferences.
   */
  void refreshSettings (void);

  virtual void resizeEvent (QResizeEvent*);
//virtual void setContentsPos (int x, int y);
//virtual void resizeContents (int w, int h);

signals:
  void activated (void);

protected:
  /** Accepts URL drags.
   */
  virtual bool acceptDrag (QDropEvent*) const;
  /** Handles the drop event.
   */
  virtual void contentsDropEvent (QDropEvent*);
  virtual void focusOutEvent (QFocusEvent*);
  virtual void focusInEvent (QFocusEvent*);
  virtual void windowActivationChange (bool);

  /** Reloads the current playlist if it was temporary.
    */
  void reload (void);
  /** Makes enough room for the given number of new items.
   */
  bool makeRoom (int, KPlayerPlaylistViewItem* = 0);
  /** Updates the playlist actions.
   */
  virtual void showEvent (QShowEvent*);
  /** Updates the playlist actions.
   */
  virtual void hideEvent (QHideEvent*);

  /** Enables or disables the player shortcuts to avoid clashes with the line edit.
   */
  void enablePlayerShortcuts (bool);

  bool m_locked, m_selection_only, m_shortcuts_enabled;
  bool m_enable_play, m_enable_pause, m_enable_stop;
  KPlayerPlaylist* m_list;
  KActionCollection* m_ac;
};

/**The KPlayer playlist window.
  *@author kiriuja
  */
class KPlayerPlaylistWindow : public QDockWindow
{
   Q_OBJECT

public: 
  KPlayerPlaylistWindow (QWidget* parent = 0, const char* name = 0);

  KPlayerPlaylistWidget* kPlayerPlaylistWidget (void)
    { return m_playlist_widget; }

protected:
  //virtual void showEvent (QShowEvent*);
  virtual void hideEvent (QHideEvent*);
  virtual void focusOutEvent (QFocusEvent*);
  virtual void focusInEvent (QFocusEvent*);
  virtual void windowActivationChange (bool);

  KPlayerPlaylistWidget* m_playlist_widget;

signals:
  void windowHidden (void);
};

#endif
