/***************************************************************************
                          kplayerprocess.h  -  description
                          ----------------
    begin                : Sat Jan 11 2003
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KPLAYERPROCESS_H
#define KPLAYERPROCESS_H

#include <kprocess.h>
#include <kurl.h>
#include <qobject.h>

#include "kplayersettings.h"

/**KProcess extension, handles line output.
  *@author kiriuja
  */

class KPlayerLineOutputProcess : public KProcess
{
  Q_OBJECT

public:
  /** Single character line terminators.
    */
//  enum Terminator
//  {
    /** No terminator. The last line of the process output was not terminated.
     */
//    None,
    /** Carriage return.
     */
//    CR,
    /** Line feed.
     */
//    LF
//  };

  /** Constructs the objects and sets the initial property values.
    */
  KPlayerLineOutputProcess (void);
  /** Frees up the allocated memory.
    */
  virtual ~KPlayerLineOutputProcess();

  /** Returns the merge setting that indicates whether stdout and stderr output are handled
      separately or merged. If they are merged, receivedStderrLine will not be emitted. */
  bool merge (void)
    { return m_merge; }
  /** Sets the merge option that specifies whether stdout and stderr output are handled
      separately or merged. If they are merged, receivedStderrLine will not be emitted. */
  void setMerge (bool merge)
    { m_merge = merge; }

protected slots:
  /** Handles the process stdout output. Emits the receivedStdoutLine signal.
    */
  void slotReceivedStdout (KProcess*, char*, int);
  /** Handles the process stderr output. Emits the receivedStderrLine signal.
    */
  void slotReceivedStderr (KProcess*, char*, int);
  /** Reimplemented from KProcess. Emits receivedStd*Line signals
      if the respective buffers are not empty. */
  virtual void processHasExited (int);

signals:
  /** Emitted when a CR and/or LF terminated line is received on stdout
      or the process finishes and the stdout buffer is not empty. */
  void receivedStdoutLine (KPlayerLineOutputProcess*, char*, int);
  /** Emitted when a CR and/or LF terminated line is received on stderr
      or the process finishes and the stderr buffer is not empty. */
  void receivedStderrLine (KPlayerLineOutputProcess*, char*, int);

protected:
  /** Handles the process output, splits it into lines and emits the appropriate signals.
   */
  void receivedOutput (KProcess* proc, char* str, int len, char* buf, int blen, int llen, bool stdout);

  /** Current stdout line or part thereof.
    */
  char* m_stdout_buffer;
  /** Current stdout buffer length.
    */
  int m_stdout_buffer_length;
  /** Current stdout line length.
    */
  int m_stdout_line_length;
  /** Current stderr line or part thereof.
    */
  char* m_stderr_buffer;
  /** Current stderr buffer length.
    */
  int m_stderr_buffer_length;
  /** Current stderr line length.
    */
  int m_stderr_line_length;
  /** Specifies whether stdout and stderr output are handled separately or merged.
      If they are merged, all output will be reported via receivedStdoutLine, and
      receivedStderrLine will not be emitted. */
  bool m_merge;
};

/**Handles the MPlayer process invocation, input and output.
  *@author kiriuja
  */
class KPlayerProcess : public QObject
{
  Q_OBJECT

public: 
  /** The process states.
    */
  enum State
  {
    /** mplayer process is not running
      */
    Idle,
    /** mplayer process is running but has not started playing yet
      */
    Running,
    /** mplayer is playing the current file
      */
    Playing,
    /** mplayer is paused
      */
    Paused
  };

  /** The frame dropping options.
    */
  enum FrameDropping
  {
    /** no frame dropping
      */
    None,
    /** soft frame dropping
      */
    Soft,
    /** hard frame dropping
      */
    Hard
  };

  /** The progress type values.
    */
  enum ProgressType
  {
    /** position progress
      */
    Position,
    /** cache fill progress
      */
    CacheFill,
    /** index generation progress
      */
    IndexGeneration
  };

  /** Constructs the objects and sets the initial property values.
    */
  KPlayerProcess (void);
  /** Frees up the allocated memory.
    */
  virtual ~KPlayerProcess();

  /** Returns the current file url or path.
    */
  KURL url (void) const
    { return m_url; }
  /** Sets the current file url or path. Autoloads the subtitle file if it exists.
    */
  void setUrl (KURL);
  /** Returns the current subtitle file url or path.
    */
  KURL subtitleUrl (void) const
    { return m_subtitle; }
  /** Sets the current subtitle file url or path.
    */
  void setSubtitleUrl (KURL url)
    { m_subtitle = url; }
  int winId (void) const
    { return m_winId; }
  /** Sets the window id where video should be output.
    */
  void setWinId (int id)
    { m_winId = id; }

  /** Retrieves information on the current file.
    */
  bool get_info (void);
  /** Runs mplayer on the current file.
    */
  bool play (void);
  /** Pauses playback.
    */
  void pause (void);
  /** Stops playback.
    */
  void stop (void);
  /** Detaches the mplayer process.
    */
  void kill (void);
  /** The current process state.
    */
  KPlayerProcess::State state (void) const
    { return m_state; }
  /** Move to the time position given.
    */
  void absoluteSeek (int);
  /** Move by the time difference given.
    */
  void relativeSeek (int);
  /** Current volume level.
    */
  int volume (void) const
    { return m_volume; }
  /** Changes volume level.
    */
  void setVolume (int);
  /** Turns the sound off.
    */
  void mute (void);
  /** Current position in seconds.
    */
  float position (void) const
    { return m_position; }
  /** Current position as string.
    */
  QString positionString (void) const;
  /** Time length in seconds.
    */
  float length (void) const
    { return m_length; }
  /** Time length as string.
    */
  QString lengthString (void) const
    { return timeString (length()); }
  /** Video width.
    */
  int width (void) const
    { return m_width; }
  /** Video height.
    */
  int height (void) const
    { return m_height; }
  /** Video size.
    */
  QSize size (void) const
    { return QSize (m_width, m_height); }
  /** Frame dropping setting.
    */
  KPlayerProcess::FrameDropping frameDropping (void) const
    { return m_frame_drop; }
  /** Changes the frame dropping setting.
    */
  void setFrameDropping (KPlayerProcess::FrameDropping);
  /** Current brightness level.
    */
  int brightness (void) const
    { return m_brightness; }
  /** Changes brightness level.
    */
  void setBrightness (int);
  /** Current contrast level.
    */
  int contrast (void) const
    { return m_contrast; }
  /** Changes contrast level.
    */
  void setContrast (int);
  /** Current hue level.
    */
  int hue (void) const
    { return m_hue; }
  /** Changes hue level.
    */
  void setHue (int);
  /** Current saturation level.
    */
  int saturation (void) const
    { return m_saturation; }
  /** Changes saturation level.
    */
  void setSaturation (int);
  /** Returns whether subtitles are visible.
    */
  bool showSubtitles (void) const
    { return m_show_subtitles; }
  /** Shows or hides subtitles.
    */
  void setShowSubtitles (bool);
  /** Moves subtitles vertically.
   */
  void subtitleMove (int);
  /** Adjusts subtitle delay.
   */
  void subtitleDelay (int);
  /** The video codec.
   */
  QString videoCodec (void)
    { return m_video_codec; }
  /** The video bitrate.
   */
  int videoBitrate (void)
    { return m_video_bitrate; }
  /** The audio codec.
   */
  QString audioCodec (void)
    { return m_audio_codec; }
  /** The audio bitrate.
   */
  int audioBitrate (void)
    { return m_audio_bitrate; }
  /** Returns whether the file information is available.
   */
  bool isInfoAvailable (void)
    { return m_info_available; }
  /** Returns whether a 0.9x version of MPlayer was detected.
   */
  bool is09Version (void)
    { return m_09_version; }

protected:
  /** The pointer to the mplayer process object.
    */
  KPlayerLineOutputProcess* m_player;
  /** The pointer to the mplayer process used to get file info.
    */
  KPlayerLineOutputProcess* m_helper;

  // Following are options passed to mplayer
  /** The url or path of the current file.
    */
  KURL m_url;
  /** The url or path of the current subtitle file.
    */
  KURL m_subtitle;
  /** Window id for video output.
    */
  int m_winId;
  /** Frame dropping.
    */
  KPlayerProcess::FrameDropping m_frame_drop;
  /** Video width.
    */
  int m_width;
  /** Video height.
    */
  int m_height;
  /** Sound volume.
    */
  int m_volume;
  /** Sound muted.
    */
  bool m_mute;
  /** Video brightness.
    */
  int m_brightness;
  /** Video contrast.
    */
  int m_contrast;
  /** Video hue.
    */
  int m_hue;
  /** Video saturation.
    */
  int m_saturation;
  /** Subtitle visibility.
   */
  bool m_show_subtitles;
  /** Time length.
    */
  float m_length;
  /** Current position.
    */
  float m_position;
  /** Current state.
    */
  KPlayerProcess::State m_state;
  /** Video codec.
   */
  QString m_video_codec;
  /** Video bitrate.
   */
  int m_video_bitrate;
  /** Audio codec.
   */
  QString m_audio_codec;
  /** Audio bitrate.
   */
  int m_audio_bitrate;
  /** Indicates whether the file information is available.
   */
  bool m_info_available;
  /** Indicates if a 0.9x version of MPlayer was detected.
   */
  bool m_09_version;

  /** Prepares and runs the given process.
    */
  bool run (KPlayerLineOutputProcess* player);
  /** Sets the process state and emits the stateChanged signal.
    */
  void setState (KPlayerProcess::State);

  // Following should really be private
  int m_seek, m_seek_count, m_ans_length;
  float m_helper_position, m_max_position, m_paused_position;
  bool m_paused, m_quit, m_size_sent, m_delayed_play;

  /** Converts a string to a floating point number, replacing a comma with a decimal point.
   */
  static float stringToFloat (QString);

protected slots:
  /** Receives notification when the mplayer process exits.
    */
  void onProcessExited (KProcess*);
  /** Receives notification when mplayer sends something to stdout.
    */
  void receivedStdoutLine (KPlayerLineOutputProcess*, char*, int);
  /** Receives notification when the mplayer helper process sends something to stdout.
    */
  void receivedHelperLine (KPlayerLineOutputProcess*, char*, int);
  /** Receives notification when mplayer sends something to stderr.
    */
//void receivedStderrLine (KPlayerLineOutputProcess*, char*, int);

signals:
  /** Emitted when KPlayer state has changed.
    */
  void stateChanged (KPlayerProcess::State);
  /** Emitted when mplayer progress needs to be updated.
    */
  void progressChanged (float, KPlayerProcess::ProgressType);
  /** Emitted when mplayer process used to get file information has finished.
    */
  void infoAvailable (void);
  /** Emitted when the video size becomes known.
    */
  void sizeAvailable (void);
  /** Emitted when a message is received from the mplayer process.
   */
  void messageReceived (const char*);
  /** Emitted when MPlayer fails to play a file.
   */
  void errorDetected (void);
};

#endif
